@@ -87,8 +87,9 @@ class Node(object):
     def linkModule(self, module,group, topic):
         """
         EXPERIMENTAL
-        Link a python object to the messaging service
-        This makes every method of the object accessable as callbacks over the btNexus
+        Link a python module to the messaging service
+        This makes every routine of the module accessable as callbacks over the btNexus
+        the routines are checked with inspect.isroutine()
 
         :param module: the module to be linked
         :type module: Object
@@ -102,7 +103,7 @@ class Node(object):
         funcs = [o for o in getmembers(module) if isroutine(o[1]) and not o[0].startswith('__')] # contains all functions if it is a module and all methods if it is an object except for the ones starting with a double underscore
         # tuples of the name and the function
         for funcName, func in funcs:
-            self.subscribe(group, topic, func) #funcName?
+            self.subscribe(group, topic, func, funcName)
             
 
     def subscribe(self, group, topic, callback, funcName=None):
